import 'package:article_flutter_riverpod/widgets/space.dart';
import 'package:flutter/material.dart';
import 'package:flutter_riverpod/flutter_riverpod.dart';

final countServiceProvider = StateNotifierProvider<CountService, int>((ref) => CountService(42)); // <1>

class CountService extends StateNotifier<int> {
  CountService(int firstValue) : super(firstValue);

  void increment() {
    state++;
  }
}

class StatefulExample extends StatelessWidget {
  const StatefulExample({Key? key}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return const ProviderScope(
      child: Counter(),
    );
  }
}

class Counter extends ConsumerStatefulWidget { // <2>
  const Counter({Key? key}) : super(key: key);

  @override
  ConsumerState<Counter> createState() => _CounterState(); // <3>
}

class _CounterState extends ConsumerState<Counter> { // <4>
  late int firstValue;

  @override
  void initState() {
    super.initState();
    firstValue = ref.read(countServiceProvider); // <5>
  }

  @override
  Widget build(BuildContext context) { // <6>
    final currentValue = ref.watch(countServiceProvider); // <7>

    return Column(
      mainAxisSize: MainAxisSize.min,
      children: [
        Text("First value: $firstValue"), // <8>
        HSpace.s,
        Text("Current value: $currentValue"),
        HSpace.m,
        ElevatedButton(
          onPressed: _onIncrement,
          child: const Text("Increment"),
        ),
      ],
    );
  }

  void _onIncrement() {
    ref.read(countServiceProvider.notifier).increment(); // <9>
  }
}
